@@ -5,7 +5,7 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 2.2
+;; Version: 2.3
 ;; Package-Requires: ((emacs "28.1") (compat "30"))
 ;; URL: https://github.com/minad/corfu
 ;; Keywords: abbrev, convenience, matching, completion, text
@@ -344,7 +344,8 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
     (undecorated . t)
     (cursor-type . nil)
     (no-special-glyphs . t)
-    (desktop-dont-save . t))
+    (desktop-dont-save . t)
+    (inhibit-double-buffering . t)) ;; Avoid display artifacts on X/Gtk builds
   "Default child frame parameters.")
 
 (defvar corfu--buffer-parameters
@@ -366,7 +367,8 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
     (fringe-indicator-alist (continuation) (truncation))
     (indicate-empty-lines . nil)
     (indicate-buffer-boundaries . nil)
-    (buffer-read-only . t))
+    (buffer-read-only . t)
+    (pixel-scroll-precision-mode . nil))
   "Default child frame buffer parameters.")
 
 (defvar corfu--mouse-ignore-map
@@ -427,7 +429,6 @@ is a prefix length override, which is t for manual completion."
       (cl-pushnew 'corfu-default (alist-get 'default face-remapping-alist))
       buffer)))
 
-(defvar x-gtk-resize-child-frames) ;; Not present on non-gtk builds
 (defvar corfu--gtk-resize-child-frames
   (let ((case-fold-search t))
      ;; XXX HACK to fix resizing on gtk3/gnome taken from posframe.el
@@ -441,6 +442,10 @@ is a prefix length override, which is t for manual completion."
                              (getenv "DESKTOP_SESSION") ""))
          'resize-mode)))
 
+;; Not present on non-gtk/non-x builds
+(defvar x-gtk-resize-child-frames)
+(defvar x-fast-protocol-requests)
+
 ;; Function adapted from posframe.el by tumashu
 (defun corfu--make-frame (frame x y width height)
   "Show current buffer in child frame at X/Y with WIDTH/HEIGHT.
@@ -452,6 +457,7 @@ FRAME is the existing frame."
   (let* ((window-min-height 1)
          (window-min-width 1)
          (inhibit-redisplay t)
+         (x-fast-protocol-requests t)
          (x-gtk-resize-child-frames corfu--gtk-resize-child-frames)
          (before-make-frame-hook)
          (after-make-frame-functions)
@@ -498,7 +504,7 @@ FRAME is the existing frame."
                           unless (equal (alist-get k is) v) collect p)))
       (when diff (modify-frame-parameters frame diff))
       ;; XXX HACK: `set-window-buffer' must be called to force fringe update.
-      (when (or diff (eq (window-buffer win) (current-buffer)))
+      (when (or diff (not (eq (window-buffer win) (current-buffer))))
         (set-window-buffer win (current-buffer)))
       ;; Disallow selection of root window (gh:minad/corfu#63)
       (set-window-parameter win 'no-delete-other-windows t)
@@ -514,6 +520,7 @@ FRAME is the existing frame."
   ;; Unparent child frame if EXWM is used, otherwise EXWM buffers are drawn on
   ;; top of the Corfu child frame.
   (when (and (bound-and-true-p exwm--connection) (frame-parent frame))
+    (redisplay t)
     (set-frame-parameter frame 'parent-frame nil))
   frame)
 
@@ -866,7 +873,7 @@ the last command must be listed in `corfu-continue-commands'."
   "Protect FUN such that errors are caught.
 If an error occurs, the FUN is retried with `debug-on-error' enabled and
 the stack trace is shown in the *Messages* buffer."
-  (static-if (>= emacs-major-version 30)
+  (static-if (fboundp 'handler-bind) ;; Available on Emacs 30
       (ignore-errors
         (handler-bind ((error #'corfu--debug))
           (funcall fun)))
